from os import path as osp

# from .basic import Meme
# from .thug import ThugMeme

DATA_FOLDER = osp.join(osp.dirname(__file__), 'data')
GLASSES_FILE = osp.join(DATA_FOLDER, 'glasses.png')
CIGAR_FILE = osp.join(DATA_FOLDER, 'cigar.png')
FONT = osp.join(DATA_FOLDER, 'font.ttf')
